#!/usr/bin/perl

#===================================================================================================================
#
#         FILE: check_basicauth_mechanism.pl
#
#        USAGE: check_basicauth_mechanism.pl -u <url> -l <username> -p <password> (-c <critical> -w <warning> -e <expect> -v)
#
#  DESCRIPTION: Authenticates against a web page using wrong basicauth credentials. If it passes with wrong credentials it 
#               will rise an error.
#
#      OPTIONS: ---
# REQUIREMENTS: LWP::UserAgent and Crypt::SSLeay if https support is required
#         BUGS:	If the webpage is not using basic auth you may get false positives 
#        NOTES: ---
#       AUTHOR: Tim Pretlove 
#      VERSION: 0.7
#      CREATED: 08-05-2012
#     REVISION: ---
#      LICENCE: GNU
#      CHANGES:
#           0.1 Status now shows header for easier debugging
#      
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#    
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#    
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
#===================================================================================================================

use strict;
use warnings;
use IO::Socket::SSL qw();
use WWW::Mechanize qw();
my $mech = WWW::Mechanize->new(ssl_opts => {
    SSL_verify_mode => IO::Socket::SSL::SSL_VERIFY_NONE,
    verify_hostname => 0, # this key is likely going to be removed in future LWP >6.04
});


use LWP::UserAgent;
use Getopt::Long;
use Time::HiRes qw(gettimeofday tv_interval);
use lib "/usr/local/nagios/libexec";
use utils qw(%ERRORS);

my ($debug, $secure, $host, $path);

GetOptions(
	'host=s'		=> \$host,
        'path=s'                => \$path,
        debug                   => \$debug,
        secure                  => \$secure
	) or HELP_MESSAGE();
	
	
sub testauth {
    my ($debug, $secure, $host, $path ) = @_;
    #my $ua = new LWP::UserAgent;
    my $ua = LWP::UserAgent->new(ssl_opts => {
    	SSL_verify_mode => IO::Socket::SSL::SSL_VERIFY_NONE,
    	verify_hostname => 0, # this key is likely going to be removed in future LWP >6.04
	});
    $ua->cookie_jar ( {} );
    $ua->requests_redirectable;
    #default timeout 5 seconds
    my $timeout = 5;
    $ua->timeout($timeout);
    my $proto = "http://";
    if ($secure)  {
        $proto = "https://";
    };
    my $url = ""; 
    if ($path) {
    	$url = $proto . $host . $path;
    } else {
	$url = $proto . $host . "/";
    };
    print "$url";
    my $req = new HTTP::Request GET => ($url);
        my $reqstatus = $req->content;
        print "$reqstatus\n";
    #commented by zauli. Instead of giving credential 
    # we request the response, and we get a 401 -> that is
    # the right result for requesting basic auth.
    #my $login = '';
    #my $passwd = '';
    #$req->authorization_basic($login,$passwd);
    my $response = $ua->request($req);
    print "Status: " . $response->status_line . "\n";
    if ($debug) {
		my $str = $response->content;
		print "$str\n";
    }
    my @str = $response->status_line;
    my $title = $str[0];
    if ($title eq "401 Authorization Required") {
		return 0, $title ;
    } else { return 1, $title } ;
}

sub HELP_MESSAGE {
	print "$0 -h <host_ip_or_name>\n";
        print "\t -p <path> # default "/"\n";
        print "\t -s use secure protocol https\n";
        print "\t -d prints page contents (debugging info)\n";
	print "\t -h <host_ip_or_name> [-p <path> -s -d] # url string to run basic auth against\n";
	print "\t e.g $0 -h www.foobar.com -s -d\n";
	exit 0;
}

sub checkopts {
    my ($debug, $secure, $host, $path) = @_;
	if (!defined $host) {
		HELP_MESSAGE();
		exit 4;
	}
}

checkopts($debug, $secure, $host, $path);

my ($rc,$title) = testauth($debug, $secure, $host, $path);
my @mess = qw(OK CRITICAL);
my @mess2 = ("Basic Authentication found"," No Basic Authentication found" );
print "HTTPAUTH $mess[$rc]: $mess2[$rc] ($title)";
print "\n";	
exit $ERRORS{$mess[$rc]};
